@@ -26,8 +26,8 @@ import stat
 import urlparse
 
 import jsonschema
-from oslo.serialization import jsonutils
 from oslo_config import cfg
+from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import units
 
@@ -116,10 +116,12 @@ class ChunkedFile(object):
     something that can iterate over a large file
     """
 
-    def __init__(self, filepath, offset=0, chunk_size=None, partial=False):
-        self.partial = partial
+    def __init__(self, filepath, offset=0, chunk_size=4096,
+                 partial_length=None):
         self.filepath = filepath
         self.chunk_size = chunk_size
+        self.partial_length = partial_length
+        self.partial = self.partial_length is not None
         self.fp = open(self.filepath, 'rb')
         if offset:
             self.fp.seek(offset)
@@ -129,12 +131,19 @@ class ChunkedFile(object):
         try:
             if self.fp:
                 while True:
-                    chunk = self.fp.read(self.chunk_size)
+                    if self.partial:
+                        size = min(self.chunk_size, self.partial_length)
+                    else:
+                        size = self.chunk_size
+
+                    chunk = self.fp.read(size)
                     if chunk:
                         yield chunk
 
                         if self.partial:
-                            break
+                            self.partial_length -= len(chunk)
+                            if self.partial_length <= 0:
+                                break
                     else:
                         break
         finally:
@@ -326,10 +335,12 @@ class Store(glance_store.driver.Store):
             for datadir in self.conf.glance_store.filesystem_store_datadirs:
                 (datadir_path,
                  priority) = self._get_datadir_path_and_priority(datadir)
-                self._check_directory_paths(datadir_path, directory_paths)
+                priority_paths = self.priority_data_map.setdefault(
+                    int(priority), [])
+                self._check_directory_paths(datadir_path, directory_paths,
+                                            priority_paths)
                 directory_paths.add(datadir_path)
-                self.priority_data_map.setdefault(int(priority),
-                                                  []).append(datadir_path)
+                priority_paths.append(datadir_path)
 
             self.priority_list = sorted(self.priority_data_map,
                                         reverse=True)
@@ -340,7 +351,8 @@ class Store(glance_store.driver.Store):
         if metadata_file:
             self._validate_metadata(metadata_file)
 
-    def _check_directory_paths(self, datadir_path, directory_paths):
+    def _check_directory_paths(self, datadir_path, directory_paths,
+                               priority_paths):
         """
         Checks if directory_path is already present in directory_paths.
 
@@ -354,9 +366,15 @@ class Store(glance_store.driver.Store):
                      "multiple times in filesystem_store_datadirs "
                      "option of filesystem configuration") %
                    {'datadir_path': datadir_path})
-            LOG.exception(msg)
-            raise exceptions.BadStoreConfiguration(
-                store_name="filesystem", reason=msg)
+
+            # If present with different priority it's a bad configuration
+            if datadir_path not in priority_paths:
+                LOG.exception(msg)
+                raise exceptions.BadStoreConfiguration(
+                    store_name="filesystem", reason=msg)
+
+            # Present with same prio (exact duplicate) only deserves a warning
+            LOG.warning(msg)
 
     def _get_datadir_path_and_priority(self, datadir):
         """
@@ -451,8 +469,8 @@ class Store(glance_store.driver.Store):
         LOG.debug(msg)
         return (ChunkedFile(filepath,
                             offset=offset,
-                            chunk_size=chunk_size or self.READ_CHUNKSIZE,
-                            partial=chunk_size is not None),
+                            chunk_size=self.READ_CHUNKSIZE,
+                            partial_length=chunk_size),
                 chunk_size or filesize)
 
     def get_size(self, location, context=None):
@@ -489,7 +507,8 @@ class Store(glance_store.driver.Store):
                 LOG.debug(_("Deleting image at %(fn)s"), {'fn': fn})
                 os.unlink(fn)
             except OSError:
-                raise exceptions.Forbidden(_("You cannot delete file %s") % fn)
+                raise exceptions.Forbidden(
+                    message=(_("You cannot delete file %s") % fn))
         else:
             raise exceptions.NotFound(image=fn)
 
