@@ -15,12 +15,9 @@
 import ConfigParser
 import logging
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 
-from oslo.config import cfg
+from oslo_config import cfg
 
 from glance_store import exceptions
 from glance_store import i18n
