@@ -19,13 +19,14 @@ import hashlib
 import httplib
 import logging
 import os
-import socket
 
 from oslo_config import cfg
 from oslo_utils import excutils
+from oslo_utils import netutils
 from oslo_utils import units
 from oslo_vmware import api
 from oslo_vmware import constants
+import oslo_vmware.exceptions as vexc
 from oslo_vmware.objects import datacenter as oslo_datacenter
 from oslo_vmware.objects import datastore as oslo_datastore
 from oslo_vmware import vim_util
@@ -94,22 +95,24 @@ _VMWARE_OPTS = [
     cfg.BoolOpt('vmware_api_insecure',
                 default=False,
                 help=_('Allow to perform insecure SSL requests to ESX/VC.')),
-    cfg.MultiStrOpt('vmware_datastores',
-                    help=_('The datastores where the images are stored inside '
-                           'vCenter. The expected format is '
-                           'datacenter_path:datastore_name:weight. The weight '
-                           'will be used unless there is not enough free '
-                           'space to store the image. If the weights are '
-                           'equal, the datastore with most free space '
-                           'is chosen.'))]
-
-
-def is_valid_ipv6(address):
-    try:
-        socket.inet_pton(socket.AF_INET6, address)
-        return True
-    except Exception:
-        return False
+    cfg.MultiStrOpt(
+        'vmware_datastores',
+        help=_(
+            'A list of datastores where the image can be stored. This option '
+            'may be specified multiple times for specifying multiple '
+            'datastores. Either one of vmware_datastore_name or '
+            'vmware_datastores is required. The datastore name should be '
+            'specified after its datacenter path, seperated by ":". An '
+            'optional weight may be given after the datastore name, seperated '
+            'again by ":". Thus, the required format becomes '
+            '<datacenter_path>:<datastore_name>:<optional_weight>. When '
+            'adding an image, the datastore with highest weight will be '
+            'selected, unless there is not enough free space available in '
+            'cases where the image size is already known. If no weight is '
+            'given, it is assumed to be zero and the directory will be '
+            'considered for selection last. If multiple datastores have the '
+            'same weight, then the one with the most free space available is '
+            'selected.'))]
 
 
 def http_response_iterator(conn, response, size):
@@ -208,7 +211,7 @@ class StoreLocation(location.StoreLocation):
         self.query = urlparse.urlencode(param_list)
 
     def get_uri(self):
-        if is_valid_ipv6(self.server_host):
+        if netutils.is_valid_ipv6(self.server_host):
             base_url = '%s://[%s]%s' % (self.scheme,
                                         self.server_host, self.path)
         else:
@@ -257,21 +260,16 @@ class Store(glance_store.Store):
                      capabilities.BitMasks.DRIVER_REUSABLE)
     OPTIONS = _VMWARE_OPTS
     WRITE_CHUNKSIZE = units.Mi
-    # FIXME(arnaud): re-visit this code once the store API is cleaned up.
-    _VMW_SESSION = None
 
     def __init__(self, conf):
         super(Store, self).__init__(conf)
         self.datastores = {}
 
-    def reset_session(self, force=False):
-        if Store._VMW_SESSION is None or force:
-            Store._VMW_SESSION = api.VMwareAPISession(
-                self.server_host, self.server_username, self.server_password,
-                self.api_retry_count, self.tpoll_interval)
-        return Store._VMW_SESSION
-
-    session = property(reset_session)
+    def reset_session(self):
+        self.session = api.VMwareAPISession(
+            self.server_host, self.server_username, self.server_password,
+            self.api_retry_count, self.tpoll_interval)
+        return self.session
 
     def get_schemes(self):
         return (STORE_SCHEME,)
@@ -305,7 +303,7 @@ class Store(glance_store.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name='vmware_datastore', reason=msg)
 
-    def configure(self):
+    def configure(self, re_raise_bsc=False):
         self._sanity_check()
         self.scheme = STORE_SCHEME
         self.server_host = self._option_get('vmware_server_host')
@@ -314,7 +312,8 @@ class Store(glance_store.Store):
         self.api_retry_count = self.conf.glance_store.vmware_api_retry_count
         self.tpoll_interval = self.conf.glance_store.vmware_task_poll_interval
         self.api_insecure = self.conf.glance_store.vmware_api_insecure
-        super(Store, self).configure()
+        self.session = self.reset_session()
+        super(Store, self).configure(re_raise_bsc=re_raise_bsc)
 
     def _get_datacenter(self, datacenter_path):
         search_index_moref = self.session.vim.service_content.searchIndex
@@ -444,7 +443,7 @@ class Store(glance_store.Store):
     def _build_vim_cookie_header(self, verify_session=False):
         """Build ESX host session cookie header."""
         if verify_session and not self.session.is_current_session_active():
-            self.reset_session(force=True)
+            self.reset_session()
         vim_cookies = self.session.vim.client.options.transport.cookiejar
         if len(list(vim_cookies)) > 0:
             cookie = list(vim_cookies)[0]
@@ -579,6 +578,10 @@ class Store(glance_store.Store):
             datacenter=dc_obj.ref)
         try:
             self.session.wait_for_task(delete_task)
+        except vexc.FileNotFoundException:
+            msg = _('Image file %s not found') % file_path
+            LOG.warn(msg)
+            raise exceptions.NotFound(message=msg)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE('Failed to delete image %(image)s '
@@ -605,7 +608,7 @@ class Store(glance_store.Store):
                                                      location.image_id})
             if resp.status >= 400:
                 if resp.status == httplib.UNAUTHORIZED:
-                    self.reset_session(force=True)
+                    self.reset_session()
                     continue
                 if resp.status == httplib.NOT_FOUND:
                     reason = _('VMware datastore could not find image at URI.')
