@@ -21,6 +21,7 @@ import uuid
 import mock
 from oslo_utils import units
 from oslo_vmware import api
+from oslo_vmware.exceptions import FileNotFoundException
 from oslo_vmware.objects import datacenter as oslo_datacenter
 from oslo_vmware.objects import datastore as oslo_datastore
 import six
@@ -94,7 +95,8 @@ class TestStore(base.StoreBaseTest,
                 test_store_capabilities.TestStoreCapabilitiesChecking):
 
     @mock.patch.object(vm_store.Store, '_get_datastore')
-    def setUp(self, mock_get_datastore):
+    @mock.patch('oslo_vmware.api.VMwareAPISession')
+    def setUp(self, mock_api_session, mock_get_datastore):
         """Establish a clean test environment."""
         super(TestStore, self).setUp()
 
@@ -224,6 +226,19 @@ class TestStore(base.StoreBaseTest,
             HttpConn.return_value = FakeHTTPConnection(status=404)
             self.assertRaises(exceptions.NotFound, self.store.get, loc)
 
+    @mock.patch('oslo_vmware.api.VMwareAPISession')
+    def test_delete_non_existing(self, mock_api_session):
+        """
+        Test that trying to delete an image that doesn't exist raises an error
+        """
+        loc = location.get_location_from_uri(
+            "vsphere://127.0.0.1/folder/openstack_glance/%s?"
+            "dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
+        with mock.patch.object(self.store.session,
+                               'wait_for_task') as mock_task:
+            mock_task.side_effect = FileNotFoundException
+            self.assertRaises(exceptions.NotFound, self.store.delete, loc)
+
     @mock.patch('oslo_vmware.api.VMwareAPISession')
     def test_get_size(self, mock_api_session):
         """
@@ -424,21 +439,11 @@ class TestStore(base.StoreBaseTest,
 
     @mock.patch.object(api, 'VMwareAPISession')
     def test_reset_session(self, mock_api_session):
-        # Initialize session and reset mock before testing.
         self.store.reset_session()
-        mock_api_session.reset_mock()
-        self.store.reset_session(force=False)
-        self.assertFalse(mock_api_session.called)
-        self.store.reset_session()
-        self.assertFalse(mock_api_session.called)
-        self.store.reset_session(force=True)
         self.assertTrue(mock_api_session.called)
 
     @mock.patch.object(api, 'VMwareAPISession')
     def test_build_vim_cookie_header_active(self, mock_api_session):
-        # Initialize session and reset mock before testing.
-        self.store.reset_session()
-        mock_api_session.reset_mock()
         self.store.session.is_current_session_active = mock.Mock()
         self.store.session.is_current_session_active.return_value = True
         self.store._build_vim_cookie_header(True)
@@ -446,9 +451,6 @@ class TestStore(base.StoreBaseTest,
 
     @mock.patch.object(api, 'VMwareAPISession')
     def test_build_vim_cookie_header_expired(self, mock_api_session):
-        # Initialize session and reset mock before testing.
-        self.store.reset_session()
-        mock_api_session.reset_mock()
         self.store.session.is_current_session_active = mock.Mock()
         self.store.session.is_current_session_active.return_value = False
         self.store._build_vim_cookie_header(True)
@@ -456,9 +458,6 @@ class TestStore(base.StoreBaseTest,
 
     @mock.patch.object(api, 'VMwareAPISession')
     def test_build_vim_cookie_header_expired_noverify(self, mock_api_session):
-        # Initialize session and reset mock before testing.
-        self.store.reset_session()
-        mock_api_session.reset_mock()
         self.store.session.is_current_session_active = mock.Mock()
         self.store.session.is_current_session_active.return_value = False
         self.store._build_vim_cookie_header()
